#include <iostream>
#include <fstream>  // std::ofstream, std::ifstream (FileStreams)
#include <ios>      // std::ios::binary
#include <string>

// Note we receive a reference to std::ostream, not std::ofstream.
// This is OK, because of inheritance and polymorphism.
// std::ofstream is a child class of std::ostream, and supports all of the
// operations of the std::ostream class.
void write_chars(std::ostream& os) {
  // four 1-byte integers
  char byte1 = 72;
  char byte2 = 105;
  char byte3 = 33;
  char byte4 = 10;
  
  // write each one to the output stream
  os.write(&byte1, 1);
  os.write(&byte2, 1);
  os.write(&byte3, 1);
  os.write(&byte4, 1);
}

void write_shorts(std::ostream& os) {
  // two 2-byte integers
  short word1 = 26952;
  short word2 = 2593;
  
  // write each one to the output stream
  // must convince the compiler to send the address
  // of a short to a function parameter that is supposed to
  // receive the address of a char.
  // let the write method know that 2 bytes should be written.
  os.write((char *) &word1, 2);
  os.write((char *) &word2, 2);
}

void write_ints(std::ostream& os) {
  // one 4-byte integer
  int word1 = 169961800;
  
  // write each one to the output stream.
  // must convince the compiler to send the address
  // of a int to a function parameter that is supposed to
  // receive the address of a char.
  // let the write method know that 4 bytes should be written.
  os.write((char *) &word1, 4);
}

void write_string(std::ostream& os) {
  // use the overloaded << operator that will
  // send bytes to represent the information in
  // text.
  os << "Hi!" << std::endl;
}

int main() {
  // Get filename from user
  std::string filename;
  std::cout << "Filename? ";
  std::cin >> filename;


  // OutputFileStream, in Binary mode
  std::ofstream data_file(filename, std::ios::binary);
  write_chars(data_file);
  write_shorts(data_file);
  write_ints(data_file);
  write_string(data_file);
  data_file.close();

  return 0;
}
