C++ References
==============

The use of `&` can be confusing in C++, until
you follow a couple of rules.  The main problem
is that the meaning of `&` is contextual.

Rule 1
------

When `&` is used in a variable declaration,
the variable is not new memory space, but
a reference to the variable it is initialized
to.  The implications:

- This variable is really just an alias for
  the variable it is initialized to.
- `const` reference variables are used
  to reference `const` references, or references
  that should not be changed via the reference.

Rule 2
------

When `&` is used in the parameter list of
a function declaration or definition, it always 
means that the parameter is a reference to the 
variable used in the function call.  The 
implications are:

- The data of the calling argument is not
  copied to a new location in memory.
- The call is usually faster because
  the data isn't copied.
- The called function can change the
  value of the variable, and the
  variable in the calling function
  will be changed.
- To gain the speed up, but to disallow
  the modification, `const` references
  are used.

  
Rule 3
------

When `&` is used in the return type
of a function, it always means that
a reference to the variable used
in the return statement is returned,
instead of a copy of its value.
The implications are:

- The data of the argument to return is
  not copied.
- The return is usually faster, because
  the data doesn't have to be copied.
- The calling function has access to the
  variable that was available inside the
  called function.
- If the returned variable is a stack
  allocated variable, it will be deallocated
  automatically when the function returns,
  so a reference to it will be a bad reference.
  The compiler doesn't protect programmers
  from this error.
- Good use cases are for references to static
  variables or data members to be returned.
- To protect from the calling function's code
  changing the contents of the returned referenced
  variable `const` references are returned.


Rule 4
------

When `&` is used on a single variable in an 
expression, it is the address operator, and 
the value calculated is the address of the 
operand variable.

