#include "prime_functions.h"
#include <vector>
#include <iostream>
#include <sstream>
#include <chrono>
#include <thread>
#include <mutex>


void prime_counter_worker(std::vector<int>& tasks, int& count, std::mutex& counter_lock) {
  while(true) {

    counter_lock.lock();
    if(tasks.size() == 0) { 
      counter_lock.unlock();
      break; 
    }
    int n = tasks.back();
    tasks.pop_back();
    counter_lock.unlock();

    if(is_prime(n)) {
      // collect it
      counter_lock.lock();
      count++; // shared variable
      counter_lock.unlock();
    }
  }
}

int main(int argc, char **argv) {
  // check every number and collect the prime numbers
  int N = 1;
  int thread_count = 2;
  if(argc > 1) {
    // argv[1] is the argument

    // turn the string into a stream
    std::stringstream ss;
    ss.str(argv[1]);

    // read from the stream
    ss >> N;
  }
  if(argc > 2) {
    // argv[2] is the argument

    // turn the string into a stream
    std::stringstream ss;
    ss.str(argv[2]);

    // read from the stream
    ss >> thread_count;
  }

  std::vector<int> tasks;
  std::mutex counter_lock;
  std::vector<int> found_primes;
  int count = 0;
  std::chrono::steady_clock::time_point t0 = std::chrono::steady_clock::now();
  std::vector<std::thread> threads;
  int i;
  tasks.resize(N+1);
  for(i = 0; i <= N; i++) {
    tasks[i] = i;
  }
  for(i = 0; i < thread_count; i++) {
    threads.push_back( std::thread(prime_counter_worker, std::ref(tasks), std::ref(count), std::ref(counter_lock)) );
  }
  for(i = 0; i < thread_count; i++) {
    threads[i].join();
  }

  std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
  std::chrono::duration<double> dt = std::chrono::duration_cast<std::chrono::duration<double>>(t1-t0);

  std::cout << count << " primes found from 0 to " << N << ", in " << dt.count() << " seconds." << std::endl;

  return 0;
}
