#include "prime_functions.h"
#include <vector>
#include <iostream>
#include <sstream>
#include <chrono>
#include <thread>


void prime_counter_worker(const int& first, const int& last, int& count) {
  int n;
  for(n = first; n < last; n++) {
    if(is_prime(n)) {
      // collect it
      count++;
    }
  }
}

int main(int argc, char **argv) {
  // check every number and collect the prime numbers
  int N = 1;
  int thread_count = 2;
  if(argc > 1) {
    // argv[1] is the argument

    // turn the string into a stream
    std::stringstream ss;
    ss.str(argv[1]);

    // read from the stream
    ss >> N;
  }
  if(argc > 2) {
    // argv[2] is the argument

    // turn the string into a stream
    std::stringstream ss;
    ss.str(argv[2]);

    // read from the stream
    ss >> thread_count;
  }

  std::vector<int> found_primes;
  int count = 0;
  std::chrono::steady_clock::time_point t0 = std::chrono::steady_clock::now();
  std::vector<std::thread> threads;
  int i;
  for(i = 0; i < thread_count; i++) {
    int first = i*(N/thread_count);
    int last = (i+1)*(N/thread_count);
    threads.push_back( std::thread(prime_counter_worker, first, last, std::ref(count)) );
  }

  for(i = 0; i < thread_count; i++) {
    threads[i].join();
  }

  std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
  std::chrono::duration<double> dt = std::chrono::duration_cast<std::chrono::duration<double>>(t1-t0);

  std::cout << count << " primes found from 0 to " << N << ", in " << dt.count() << " seconds." << std::endl;

  return 0;
}
