#include "PrimeCounter.h"
#include "prime_functions.h"
#include <thread>
#include <vector>

PrimeCounter::PrimeCounter() 
  : mCount(0) {
}

void PrimeCounter::fill_tasks(int N) {
  int i;
  mTasks.resize(N+1);
  for(i = 0; i <= N; i++) {
    mTasks[i] = i;
  }
}

void PrimeCounter::process_all_tasks(int thread_count) {
  std::vector<std::thread> threads;
  int i;
  for(i = 0; i < thread_count; i++) {
    threads.push_back( std::thread(&PrimeCounter::prime_counter_worker, this) );
    // this->prime_counter_worker()
  }
  for(i = 0; i < thread_count; i++) {
    threads[i].join();
  }
}

void PrimeCounter::prime_counter_worker() {
  while(true) {

    mCounterLock.lock();
    if(mTasks.size() == 0) { 
      mCounterLock.unlock();
      break; 
    }
    int n = mTasks.back();
    mTasks.pop_back();
    mCounterLock.unlock();

    if(is_prime(n)) {
      // collect it
      mCounterLock.lock();
      mCount++; // shared variable
      mCounterLock.unlock();
    }
  }
}

int PrimeCounter::getCount() const {
  return mCount;
}
